% Monthly 18-Apr-2020 

% trend computations and plotting, for individual stations and one
%   baywide mean
% treats shallow & deep depths individually and their mean
% does temp, sal, and strat

% all calcs use monthly values

% includes trend detection "n*" calcs
% W98 is weatherhead et al 1998 method
% L08P is Leroy et al 2008 method using Phoja method of handling ACF

% basic initial setup
wipe;
modprelims;

% retrieve inputs
ingrps = { 'Config'; }; inputsdiary;

nprm = length(prms);

for iprm = 1:nprm

    load(infil,prms{iprm},'yrs','stns','monst');
    eval(sprintf('rawdat = %s;',prms{iprm}));
    eval(sprintf('clear %s;',prms{iprm}));
    
    if iprm<3
        nyr = size(rawdat.sh,2);
        nmon = size(rawdat.sh,3);
    else
        nyr = size(rawdat,2);
        nmon = size(rawdat,3);
    end
        
    %% compute bay-wide case (for strat, only sh will be used)
    bw.sh = nan*ones(nyr,nmon);
    bw.dp = nan*ones(nyr,nmon);
    bw.bth = nan*ones(nyr,nmon);
    for iyr = 1:nyr
        for imon = 1:nmon
            if ~isequal(prms{iprm},'strat')
                tmp = squeeze(rawdat.sh(:,iyr,imon));
                if sum(~isnan(tmp))>6 % at least 6 non-nan stations
                    bw.sh(iyr,imon) = nanmean(tmp);
                end
                tmp = squeeze(rawdat.dp(:,iyr,imon));
                if sum(~isnan(tmp))>6
                    bw.dp(iyr,imon) = nanmean(tmp);
                end
                if ~isnan(bw.sh(iyr,imon)) & ~isnan(bw.dp(iyr,imon))
                    bw.bth(iyr,imon) = mean([bw.sh(iyr,imon) bw.dp(iyr,imon)]);
                end
            else
                tmp = squeeze(rawdat(:,iyr,imon));
                if sum(~isnan(tmp))>6
                    bw.sh(iyr,imon) = nanmean(tmp);
                end
            end
            clear tmp;
        end
    end
    clear iyr imon;
    stns{end+1} = 'BayWide';
    nstn = length(stns);

    % create time values (no nans)
    t = yrs+(monst/12)';
    t = t(:)';

    % initialize results containers
    stats.nyr = nan*ones(nstn,3);
    stats.alln = nan*ones(nstn,3);
    stats.a = nan*ones(nstn,3);
    stats.acf1sig = nan*ones(nstn,3);
    linmdl.slope = nan*ones(nstn,3);
    linmdl.slopeconf = nan*ones(nstn,3);
    linmdl.r2 = nan*ones(nstn,3);
    linmdl.p = nan*ones(nstn,3);
    W98.b = nan*ones(nstn,3);
    W98.nstar = nan*ones(nstn,3);
    W98.slopeunc = nan*ones(nstn,3);
    L08P = W98;

    % loop to do and save calcs, and plot sh/dp timeseries/acl results
    for istn = 1:nstn
        fprintf('Stn %s (%d/%d)\n',stns{istn},istn,nstn);
        figport('on');
        if ~isequal(prms{iprm},'strat')
            alliz = 1:3;
        else
            alliz= 1;
        end
        for iz = alliz % 1 is shallow 2 is deep
            if iz==1 str = 'sh'; elseif iz==2 str = 'dp'; else str = 'bth'; end
            fprintf('  Depth: %s (%d/%d)\n',str,iz,3);
            %% raw and climatology (long-term mean of each indiv month)
            if istn<nstn
                if ~isequal(prms{iprm},'strat')
                    eval(sprintf('raw = squeeze(rawdat.%s(istn,:,:));',str));
                else
                    raw = squeeze(rawdat(istn,:,:));
                end
            else
                eval(sprintf('raw = bw.%s(:,:);',str));
            end
            clim = nanmean(raw); % assumes no month has too few non-nan values
            clim = repmat(clim,1,nyr);
            raw = raw';
            raw = raw(:)';
            n = sum(~isnan(raw));
            stats.alln(istn,iz) = n;
            %% deseas, trend, and noise
            deseas = raw - clim;
            mdl = fitlm(t,deseas);
            coef = mdl.Coefficients;
            trend = mdl.predict(t')';
            noise = deseas - trend;
            linmdl.slope(istn,iz) =  coef.Estimate(2);
            linmdl.slopeconf(istn,iz) = coef.SE(2);
            linmdl.r2(istn,iz) = mdl.Rsquared.Ordinary;
            linmdl.p(istn,iz) = coef.pValue(1);
            %% autocorr
            if nmon>1
                % create allt and allnoise (i.e. equispaced, all 12 mon)
                allt = yrs+((0:11)/12)';
                allnoise = nan*ones(size(allt));
                for i = 1:size(allt,2)
                    allnoise(monst,i) = noise((nmon*(i-1))+(1:nmon));
                end
                allt = allt(:)';
                allnoise = allnoise(:)';
                nd = length(allnoise);
                nlag = round(10*log10(nd));
                [acf,lags,bounds] = autocorr(allnoise,'NumLags',nlag);  
            else
                allt = (yrs+monst/12)';
                allnoise = noise';
                [acf,lags,bounds] = autocorr(allnoise);  
                nlag = length(acf)-1;
            end
            % rest of calcs 
            tnyr = length(floor(t(min(find(~isnan(raw))))):2017);
            stats.nyr(istn,iz) = tnyr;
            if abs(acf(2)) < bounds(1)
                stats.acf1sig(istn,iz) = 0;
            else
                stats.acf1sig(istn,iz) = 1;
            end
            TauW98 = (1+acf(2))/(1-acf(2));
            StdOfSlopeW98 = nanstd(noise)*sqrt(TauW98)/(tnyr^(2/3));
            MinYearsW98 = ( 3.3*nanstd(noise)*sqrt(TauW98)/ ...
                                    abs(coef.Estimate(2)) )^(2/3);
            TauL08P = sum(acf(abs(acf)>bounds(1)));
            StdOfSlopeL08P = nanstd(noise)*sqrt(TauL08P)/(tnyr^(2/3));
            MinYearsL08P = ( 3.3*nanstd(noise)*sqrt(TauL08P)/ ...
                                    abs(coef.Estimate(2)) )^(2/3);
            stats.a(istn,iz) = nanstd(noise);
            W98.b(istn,iz) = sqrt(TauW98);
            W98.nstar(istn,iz) = MinYearsW98;
            W98.slopeunc(istn,iz) = StdOfSlopeW98;
            L08P.b(istn,iz) = sqrt(TauL08P);
            L08P.nstar(istn,iz) = MinYearsL08P;
            L08P.slopeunc(istn,iz) = StdOfSlopeL08P;
            % make plot
            if iz < 3
                subplot(6,1,1+(iz-1)*3);
                hold;
                for iyr = 1:nyr
                    plot(t((nmon*(iyr-1))+(1:nmon)),...
                        raw((nmon*(iyr-1))+(1:nmon)),'g.');
                    plot(t((nmon*(iyr-1))+(1:nmon)),...
                        raw((nmon*(iyr-1))+(1:nmon)),'g-');
                    plot(t((nmon*(iyr-1))+(1:nmon)),...
                        clim((nmon*(iyr-1))+(1:nmon)),'k.');
                    plot(t((nmon*(iyr-1))+(1:nmon)),...
                        clim((nmon*(iyr-1))+(1:nmon)),'k-');
                end
                set(gca,'xlim',xlim);
                set(gca,'ylim',Tylim{iprm});
                ylabel(ylab{iprm});
                title(sprintf('%s %s // Raw(g) Clim(k) n=%d',stns{istn},str,n));
                subplot(6,1,2+(iz-1)*3);
                hold;
                for iyr = 1:nyr
                    plot(t((nmon*(iyr-1))+(1:nmon)),...
                        deseas((nmon*(iyr-1))+(1:nmon)),'r.');
                    plot(t((nmon*(iyr-1))+(1:nmon)),...
                        deseas((nmon*(iyr-1))+(1:nmon)),'r-');
                    plot(t((nmon*(iyr-1))+(1:nmon)),...
                        noise((nmon*(iyr-1))+(1:nmon)),'b.');
                    plot(t((nmon*(iyr-1))+(1:nmon)),...
                        noise((nmon*(iyr-1))+(1:nmon)),'b-');
                end
                plot(xlim,[mdl.predict(xlim(1)) mdl.predict(xlim(2))],'c');
                set(gca,'xlim',xlim);
                set(gca,'ylim',Nylim{iprm});
                ylabel([ylab{iprm} ' resid']);
                title(('Deseas(r)=Raw-Clim  Trend(c)   Noise(b)=Deseas-Trend'));
                text(xlim(1),Nylim{iprm}(2),sprintf( ...
                    ' R^{2}=%.3f p=%.3f / Slope %.2f DegC/yr (%.2f-%.2f @95%%)',...
                    mdl.Rsquared.Ordinary,coef.pValue(1),...
                    coef.Estimate(2),...
                    coef.Estimate(2)-coef.SE(2),coef.Estimate(2)+coef.SE(2)),...
                    'verticalalignment','top','fontsize',8);
                subplot(6,1,3+3*(iz-1));
                hold;
                bar(0:nlag,acf);
                set(gca,'xlim',[-0.5 nlag+0.5]);
                set(gca,'ylim',[-0.5 1]);
                plot([0 nlag],[bounds(1) bounds(1)],'b--');
                plot([0 nlag],[bounds(2) bounds(2)],'b--');
                ylabel('Noise ACF vs lag(mon)');
                x = 0.3*nlag;
                y = 1.15;
                dy = 0.19;
                text(x,y,sprintf('slope = %.3f DegC/Yr',...
                                    mdl.Coefficients.Estimate(2)));
                text(x,y-dy,sprintf(['\\sigma_{slope} W98/L08P = %.2f / %.2f '...
                    'DegC/Yr = a b / (nyr)^{2/3}'],StdOfSlopeW98,StdOfSlopeL08P));
                text(x,y-2*dy,sprintf(['n* W98/L08P = %.1f / %.1f Yr = '...
                    '[ 3.3 a b / |slope| ]^{2/3}'],MinYearsW98,MinYearsL08P));
                text(x,y-3*dy,sprintf('a = Std(noise) = %.2f DegC',nanstd(noise)));
                text(x,y-4*dy,sprintf('b = sqrt(Tau) W98/L08P = %.2f / %.2f',...
                    sqrt(TauW98),sqrt(TauL08P)));   
                clear x y dy iyr;
            end
            clear i raw clim deseas trend noise mdl n coef;
            clear allt allnoise nlag acf lags bounds;
            clear tnyr Tau* StdOf* MinYears*;
        end
        clear iz;
        eval(sprintf('print -dpng plt/%s_%s_%s_timeseries;',...
                            runname,stns{istn},prms{iprm}));
        delete(gcf);
    end
    clear istn;
    
    %% plot trend results
    % compute yaxis limits
    tr.ymax = max(max(linmdl.slope+linmdl.slopeconf));
    tr.ymin = min(min(linmdl.slope-linmdl.slopeconf));
    incr = 0.03*(tr.ymax -tr.ymin);
    tr.ymax = tr.ymax+incr;
    tr.ymin = tr.ymin-incr;
    clear incr;
    nst.ymax = max([max(max(W98.nstar)) max(max(abs(L08P.nstar)))]);
    nst.ymin = 0;
    figport('on');
    for istn = 1:nstn
        subplot(4,3,istn);
        hold;
        title(sprintf('%s nyr=%d/%d/%d',stns{istn},stats.nyr(istn,1),...
            stats.nyr(istn,2),stats.nyr(istn,3)));
        set(gca,'xlim',[0 13]);
        set(gca,'xtick',[3 7 11]);
        set(gca,'xticklabels',{'sh'; 'dp'; 'bth';});
        yyaxis left;
        bar([2 6 10],linmdl.slope(istn,:),0.8/4,'facecolor','w','edgecolor','b');
        errorbar([2 6 10],linmdl.slope(istn,:),linmdl.slopeconf(istn,:),...
            'linestyle','none','color','b');
        set(gca,'ylim',[tr.ymin tr.ymax]);
        if istn==1
            ylabel(ylab2{iprm});
        end
        yyaxis right;
        bar([3 7 11],W98.nstar(istn,:),0.8/4,'facecolor','w','edgecolor','r');
        bar([4 8 12],L08P.nstar(istn,:),0.8/4,'facecolor','w','edgecolor','r');
        plot([0 13],stats.nyr(istn,1)*ones(1,2),'r');
        set(gca,'ylim',[nst.ymin nst.ymax]);
        if istn==3
            ylabel('n*(yrs) W98/L08P lft/rgt');
        end
    end
    clear istn tr nst; 

    eval(sprintf('print -dpng plt/%s_%s_trends;',runname,prms{iprm}));
    delete(gcf);
    eval(sprintf('save dat/%s_%s rawdat bw stats linmdl W98 L08P;',...
        runname,prms{iprm}));
    clear stats linmdl W98 L08P bw rawdat;
end

% end of module
diary off;