wipe;

indir = '..\Monthly\dat\';
stns = {'PD';'BR';'CP';'NP';'GB';'SR';'MV'; 'QP';'PP';'TW';'MH';};
rownames = {'Rate';'CI95';'YrsSamp';'YrsDetect';'R2';'pval';};

%% Temp
titl = 'Temperature Trends';
infil = 'Config001_temp.mat';
units = 'deg C yr^-1';
zval = [1 2];
labls = {'TempShallow';'TempDeep';};
xlsnam = 'TempTrends.xls';

main(indir,infil,stns,rownames,titl,units,zval,labls,xlsnam);

%% Sal
titl = 'Salinity Trends';
infil = 'Config001_sal.mat';
units = 'PSU yr^-1';
zval = [1 2];
labls = {'SalShallow';'SalDeep';};
xlsnam = 'SalTrends.xls';

main(indir,infil,stns,rownames,titl,units,zval,labls,xlsnam);

%% Strat
titl = 'Stratification Trends';
infil = 'Config001_strat.mat';
units = 'kg m^-3 yr^-1';
zval = [1];
labls = {'StratShallow';'StratDeep';};
xlsnam = 'StratTrends.xls';

main(indir,infil,stns,rownames,titl,units,zval,labls,xlsnam);

% called three times
function main(indir,infil,stns,rownames,titl,units,zval,labls,xlsnam)

delete(xlsnam);
load([indir infil],'linmdl','stats','W98');

% metadata sheet
xlswrite(xlsnam,{'dan.codiga@gmail.com NBEP Contract 2018'},...
    'Metadata','A1');
xlswrite(xlsnam,{titl},...
    'Metadata','A3');
xlswrite(xlsnam,{'For NBFSMN metadata see Task 4'},'Metadata','A5');

xlswrite(xlsnam,{['Rate = slope of linear trend, units ' units]},...
    'Metadata','A7');
xlswrite(xlsnam,{['CI95 = 95% Conf. Int. on Rate (same units as Rate)'...
    ]},'Metadata','A8');
xlswrite(xlsnam,{'YrsSamp = Number of years sampled'},'Metadata','A9');
xlswrite(xlsnam,{['YrsDetect = Number of years sampling required to '...
    'detect trend (Weatherhead et al., 1998) ']},'Metadata','A10');
xlswrite(xlsnam,{['R2 = correlation coefficient squared from linear'...
    'model']},'Metadata','A11');
xlswrite(xlsnam,{'pval = p value from linear model'},'Metadata','A12');

for i = zval
    a = [linmdl.slope(1:11,i)'; linmdl.slopeconf(1:11,i)';stats.nyr(1:11,i)'; ...
        W98.nstar(1:11,i)'; linmdl.r2(1:11,i)'; linmdl.p(1:11,i)'];
    t = array2table(a);
    t.Properties.VariableNames = stns;
    t.Properties.RowNames = rownames;
    writetable(t,xlsnam,'Sheet',labls{i},'WriteRowNames',true);
    clear t a;
end

% remove Sheet1
newExcel = actxserver('excel.application');
newExcel.DisplayAlerts = false;
xls = newExcel.Workbooks.Open([pwd '\' xlsnam],0,false);
xls.Sheets.Item(1).Delete;
xls.Save();
xls.Close();
newExcel.Quit();
delete(newExcel);

% move to final destination
movefile(xlsnam,['../../DataFiles/' xlsnam]);

end
